#include "Eigen/Dense"
#include "Eigen/Sparse"
#include <iostream>
#include <cassert>
#include <chrono>


using namespace std::chrono;

template <typename T>
void run_test_op(
        const size_t &rep,
        const T &A,
        const T &B,
        T &res)
{
    for (size_t i=0; i<rep; ++i){
        res = A*B;
    }
}



template<typename D, typename S>
void compare_DenseSparse(
        const size_t &rep,
        const D &DA,
        const D &DB,
        const S &SA,
        const S &SB){

    D Dres(DA);

    auto start = high_resolution_clock::now();
    run_test_op(rep, DA, DB, Dres);
    auto stop = high_resolution_clock::now();
    auto duration = duration_cast<milliseconds>(stop-start);
    std::cout << " Time dense = " << duration.count() << std::endl;


    S Sres;

    start = high_resolution_clock::now();
    run_test_op(rep, SA, SB, Sres);
    stop = high_resolution_clock::now();
    duration = duration_cast<milliseconds>(stop-start);
    std::cout << " Time sparse = " << duration.count() << " Number of non zero entries =" << Sres.nonZeros() << std::endl;

}

int main(int argc, char* argv[]) {
  using T=double;
  using SpMat = Eigen::SparseMatrix<double>;
  using DenseMat = Eigen::Matrix<T,Eigen::Dynamic,Eigen::Dynamic>;


  assert(argc==3); int n=std::stoi(argv[1]), reps=std::stoi(argv[2]);

  std::cout << "====== Matrix size = " << n << " x " << n << "=======" << std::endl;
  for (size_t i = 0; i<3; ++i)
  {
      DenseMat A, B;
      T coeff = 3.5/(n);
      if (i != 0)
          coeff =  i*2./10;

      A=DenseMat::Random(n,n);
      B=DenseMat::Random(n,n);

      A = (abs(A.array()) < coeff).select(A,0);
      B = (abs(B.array()) < coeff).select(B,0);

      std::cout << "coeff= " << coeff << std::endl;
      std::cout << "number non zero's A= " << (A.array() != 0.0).count()
                << "   Percentage NonZero's =" <<  (A.array() != 0).count()/(double)(n*n)*100 << "%" << std::endl;
      std::cout << "number non zero's B= " << (B.array() != 0.0).count()
                << "   Percentage NonZero's =" <<  (B.array() != 0).count()/(double)(n*n)*100 << "%" << std::endl;
      SpMat SA(A.sparseView()), SB(B.sparseView());
      compare_DenseSparse(reps, A, B, SA, SB);
  }

}
