#include "Eigen/Dense"
#include "Eigen/Sparse"
#include <iostream>
#include <cassert>
#include <random>

std::default_random_engine gen;
std::uniform_real_distribution<double> dist(0.0,1.0);

using namespace Eigen;

void buildProblem(
                  std::vector<Triplet<double>> &coefficients,
                  Matrix<double,Dynamic,1> & b,
                  int &n){

  for (int i=0; i < n; ++i){
    for (int j=0; j<n; j++){
      if (i == j)
        coefficients.push_back(Triplet<double>(i,j,-4.0));
      else{
        auto v_ij = dist(gen);
        if (v_ij < 0.1)
          coefficients.push_back(Triplet<double>(i,j,v_ij));
      }
    }
  }
  b= Eigen::Matrix<double,Eigen::Dynamic,1>::Random(n);
}

void solve_LES(
        const SparseMatrix<double> &A,
        const Matrix<double,Dynamic,1> &b,
        Matrix<double,Dynamic,1> &x)
{
    SparseLU<SparseMatrix<double>, COLAMDOrdering<int> > solver;
    solver.compute(A);
    if (solver.info() != Success)
      std::cout << "Decompostion failed " << std::endl;
    x = solver.solve(b);
    if (solver.info() != Success)
      std::cout << "Solver failed " << std::endl;

}
int main(int argc, char* argv[]) {

  using SpMat = SparseMatrix<double>;
  using DenseVec = Matrix<double,Dynamic,1>;

  assert(argc==2); int n=std::stoi(argv[1]);

  std::vector<Triplet<double>> coefficients;            // list of non-zeros coefficients
  DenseVec b(n), x(n);
  buildProblem(coefficients, b, n);

  SpMat A(n,n);
  A.setFromTriplets(coefficients.begin(), coefficients.end());
  solve_LES(A,b,x);

  if ((A*x - b).norm()< 1.e-6)
      std::cout << "solution computed correctly" << std::endl;
}
