#include "Eigen/Dense"
#include "Eigen/Sparse"
#include <iostream>
#include <cassert>
#include <random>

std::default_random_engine gen;
std::uniform_real_distribution<double> dist(0.0,1.0);

using namespace Eigen;

using SpMat = SparseMatrix<double>;
using MT=Eigen::Matrix<double,Eigen::Dynamic,Eigen::Dynamic>;
using DenseVec = Matrix<double,Dynamic,1>;


void buildProblem(
        std::vector<Triplet<double>> &coefficients,
        Matrix<double,Dynamic,1> & b,
        const int &m,
        const int &n){

  for (int i=0; i < m; ++i){
    for (int j=0; j<n; j++){
      if (i == j)
        coefficients.push_back(Triplet<double>(i,j,-4.0));
      else{
        auto v_ij = dist(gen);
        if (v_ij < 0.1)
          coefficients.push_back(Triplet<double>(i,j,v_ij));
      }
    }
  }
  b= Eigen::Matrix<double,Eigen::Dynamic,1>::Random(m);
}

void Regression_QR(
                   const SparseMatrix<double> &A,
                   Matrix<double,Dynamic,1> &p,
                   const Matrix<double,Dynamic,1> &x)
{
  SparseQR<SparseMatrix<double>, COLAMDOrdering<int> > solver;
  solver.compute(A);
  if (solver.info() != Success) std::cout << "Decompostion failed " << std::endl;
  p = solver.solve(x);
  if (solver.info() != Success) std::cout << "Solver failed " << std::endl;
}

int main(int argc, char* argv[]) {


  assert(argc==3); int m=std::stoi(argv[1]); int n=std::stoi(argv[2]);

  std::vector<Triplet<double>> coefficients;
  DenseVec p(n), y(n);
  buildProblem(coefficients, y, m, n);

  SpMat A(m,n);
  A.setFromTriplets(coefficients.begin(), coefficients.end());
  Regression_QR(A,p,y);

  std::cout <<"m ="<< m <<" n=" << n <<" number non zero's of A= " << A.nonZeros()
            << "   Percentage NonZero's =" <<  A.nonZeros()/(double)(m*n)*100 << "%" << std::endl;

  std::cout << "||A*p-y|| = " << (A*p-y).squaredNorm() << std::endl;
  std::cout << "||A^T*A*p-A^T*y||=" << (A.transpose()*A*p-A.transpose()*y).norm() << std::endl;
}
